@@ -8,7 +8,7 @@ from einops import rearrange
 
 from dalle_pytorch.vae import OpenAIDiscreteVAE
 from dalle_pytorch.vae import VQGanVAE1024
-from dalle_pytorch.transformer import Transformer
+from dalle_pytorch.transformer import Transformer, ScaleNorm
 
 # helpers
 
@@ -346,7 +346,7 @@ class DALLE(nn.Module):
         )
 
         self.to_logits = nn.Sequential(
-            nn.LayerNorm(dim),
+            ScaleNorm(dim),
             nn.Linear(dim, self.total_tokens),
         )
 
@@ -372,14 +372,27 @@ class DALLE(nn.Module):
         clip = None,
         mask = None,
         filter_thres = 0.5,
-        temperature = 1.
+        temperature = 1.,
+        img = None,
+        num_init_img_tokens = None
     ):
         vae, text_seq_len, image_seq_len, num_text_tokens = self.vae, self.text_seq_len, self.image_seq_len, self.num_text_tokens
         total_len = text_seq_len + image_seq_len
 
         out = text
 
-        for cur_len in range(text.shape[1], total_len):
+        if exists(img):
+            indices = self.vae.get_codebook_indices(img)
+            num_img_tokens = default(num_init_img_tokens, int(0.4375 * image_seq_len))  # OpenAI used 14 * 32 initial tokens to prime
+            assert num_img_tokens < image_seq_len, 'number of initial image tokens for priming must be less than the total image token sequence length'
+
+            indices = indices[:, :num_img_tokens]
+            out = torch.cat((out, indices), dim = -1)
+
+            if exists(mask):
+                mask = F.pad(mask, (0, num_img_tokens), value = True)
+
+        for cur_len in range(out.shape[1], total_len):
             is_image = cur_len >= text_seq_len
 
             text, image = out[:, :text_seq_len], out[:, text_seq_len:]
